/* ------------------------------------------------------------------------- */
/* arch/ppc/mpc8xx/mpc8xx.c                                                  */
/*                                                                           */
/* mpc8xx specific code                                                      */
/*                                                                           */
/* Copyright (C) 2000 - 2001 Magnus Damm <damm@opensource.se>                */
/* Copyright (C) 1999 - 2000 Ericsson Radio Systems AB                       */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston,                     */
/* MA 02111-1307 USA                                                         */
/* ------------------------------------------------------------------------- */
/* Note that (15-1) refers to side 15-1 in MPC860UM/AD 07/98 REV.1           */
/* ------------------------------------------------------------------------- */

#include "types.h"
#include "cpu.h"
#include "board.h"
#include "console.h"
#include "8xx_immap.h"

/* ------------------------------------------------------------------------- */

void mpc8xx_udelay(unsigned int no_us)
{
  volatile cpmtimer8xx_t *t =  &(((immap_t *)mpc8xx_get_imap())->im_cpmtimer);
  unsigned int cpu_hz, dummy;
  int v, p;

  board_consoleinfo_get(&dummy, &dummy, &cpu_hz);

  v = (cpu_hz / (1000000/no_us));

  do {

    if (v < 0xffff)
      p = v;
    else
      p = 0xffff;

    v -= p;

    t->cpmt_tgcr &= 0xfff8; /* stop and reset timer (18-8) */

    t->cpmt_tmr1 = 0x001a; /* set up some values */
    t->cpmt_tcn1 = 0x0000;
    t->cpmt_trr1 = p;
    t->cpmt_ter1 = 0xffff;

    t->cpmt_tgcr |= 0x0001; /* enable timer */

    do {
      mpc8xx_watchdog_kick();
    } while (!t->cpmt_ter1) ; /* finally - poll event register */

  }  while(v);

}
  
/* ------------------------------------------------------------------------- */

void mpc8xx_set_pll(int factor)
{  
  volatile immap_t *immap = mpc8xx_get_imap();

  /* Set the PLL, the low-power modes and the reset control (15-29) */
  immap->im_clkrst.car_plprcr = ((factor-1) << 20) | 0x4000;
}

/* ------------------------------------------------------------------------- */
/* watchdog kicker                                                           */

void mpc8xx_watchdog_kick(void)
{
  volatile immap_t *immap = mpc8xx_get_imap();

  immap->im_siu_conf.sc_swsr = 0x556c;   /* (11-21) */
  immap->im_siu_conf.sc_swsr = 0xaa39;
}

/* ------------------------------------------------------------------------- */
/* start the image                                                           */

void mpc8xx_image_start(unsigned int startaddr)
{
  mpc8xx_watchdog_kick();

  mpc8xx_set_pll(1);   /* restore setting */

  mpc8xx_image_start_now(startaddr);
}

/* ------------------------------------------------------------------------- */
