/* ------------------------------------------------------------------------- */
/* arch/ppc/mpc8xx/mpc8xx_tty.c                                              */
/*                                                                           */
/* mpc8xx specific code serial port code (SMC only)                          */
/* stolen from linux/arch/ppc/mbxboot/m8xx_tty.c                             */
/*                                                                           */
/* Written by Dan Malek <dmalek@jlc.net>                                     */
/*                                                                           */
/* Modified by Magnus Damm <damm@opensource.se>                              */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston,                     */
/* MA 02111-1307 USA                                                         */
/* ------------------------------------------------------------------------- */
/* Note that (15-1) refers to side 15-1 in MPC860UM/AD 07/98 REV.1           */
/* ------------------------------------------------------------------------- */

#define RX_NO_BDS 32

#include "types.h"
#include "cpu.h"
#include "board.h"
#include "commproc.h"

#define CPMP ((cpm8xx_t *)&(((immap_t *)mpc8xx_get_imap())->im_cpm))
#define IMAP ((immap_t *)mpc8xx_get_imap())

/* this ring counter lies in the dpram, after the other serial stuff */

#define RING_CNT *((u32 *) &(CPMP->cp_dpmem[66*8]))

unsigned short const mpc8xx_tty_proff[2] = { PROFF_SMC1, PROFF_SMC2 };
unsigned short const mpc8xx_tty_cpmch[2] = { CPM_CR_CH_SMC1, CPM_CR_CH_SMC2 };

void
serial_init(u32 bps)
{
	volatile smc_t		*sp;
	volatile smc_uart_t	*up;
	volatile cbd_t	*tbdf, *rbdf;
	volatile cpm8xx_t	*cp;
	uint dpaddr;
	uint tmp, port, cpu_hz;

	board_consoleinfo_get(&port, &tmp, &cpu_hz);

	if (!bps)
	  bps = tmp;

	cp = CPMP;
	sp = (smc_t*)&(cp->cp_smc[port]);
	up = (smc_uart_t *)&cp->cp_dparam[mpc8xx_tty_proff[port]];

	/* Disable transmitter/receiver.
	*/
	sp->smc_smcmr &= ~(SMCMR_REN | SMCMR_TEN);

	/* Initialize SMC1 and use it for the console port.
	 */

	/* Enable SDMA.
	*/
	((immap_t *)IMAP)->im_siu_conf.sc_sdcr = 1;

	/* Use Port B for SMCs instead of other functions.
	*/

	if (port == 0) {
	      cp->cp_pbpar |= 0x000000c0;      /* SMC1 */
	      cp->cp_pbdir &= ~0x000000c0;
	      cp->cp_pbodr &= ~0x000000c0;
	}
	else {
	      cp->cp_pbpar |= 0x00000c00;      /* SMC2 */
	      cp->cp_pbdir &= ~0x00000c00;
	      cp->cp_pbodr &= ~0x00000c00;
	}

	/* Allocate space for two buffer descriptors in the DP ram.
	 * For now, this address seems OK, but it may have to
	 * change with newer versions of the firmware.
	 */
	dpaddr = 0;

	/* Set the physical address of the host memory buffers in
	 * the buffer descriptors.
	 */

	tbdf = (cbd_t *)&cp->cp_dpmem[dpaddr];
	tbdf->cbd_bufaddr = (uint) (tbdf+1);
	tbdf->cbd_sc = 0;

	/* Initialize all rx buffers */

	{ 
	  int k;
	  
	  for (k = 0; k < RX_NO_BDS; k++) {

	    rbdf = tbdf + 2 + k;

	    rbdf->cbd_bufaddr = ((uint) (rbdf + RX_NO_BDS));
	    rbdf->cbd_sc = BD_SC_EMPTY;
	  }

	  rbdf->cbd_sc |= BD_SC_WRAP;

	  RING_CNT = 0;
	}

	/* Set up the uart parameters in the parameter ram.
	*/
	up->smc_tbase = dpaddr;
	up->smc_rbase = dpaddr+(sizeof(cbd_t) << 1);
	up->smc_tfcr = SMC_EB;
	up->smc_rfcr = SMC_EB;

	/* Set UART mode, 8 bit, no parity, one stop.
	 * Enable receive and transmit.
	 */
	sp->smc_smcmr = smcr_mk_clen(9) |  SMCMR_SM_UART;

	/* Mask all interrupts and remove anything pending.
	*/
	sp->smc_smcm = 0;
	sp->smc_smce = 0xff;

	/* Set up the baud rate generator.
	 * See 8xx_io/commproc.c for details.
	 */

	switch (port) {
	case 0: cp->cp_simode = 0x10000000; break;
	case 1: cp->cp_simode = 0x00001000; break;
	}

	cp->cp_brgc1 = (((cpu_hz / 16) / bps) << 1) | CPM_BRG_EN;


	/* Make the first tx buffer the only buffer.
	*/
	tbdf->cbd_sc |= BD_SC_WRAP;

	/* Single character receive.
	*/
	up->smc_mrblr = 1;
	up->smc_maxidl = 0;

	/* Initialize Tx/Rx parameters.
	*/
	cp->cp_cpcr = mk_cr_cmd(mpc8xx_tty_cpmch[port], CPM_CR_INIT_TRX) 
	  | CPM_CR_FLG;

	while (cp->cp_cpcr & CPM_CR_FLG);

	/* Enable transmitter/receiver.
	*/
	sp->smc_smcmr |= SMCMR_REN | SMCMR_TEN;
}

void
serial_putc(const char c)
{
	volatile cbd_t		*tbdf;
	volatile char		*buf;
	volatile smc_uart_t	*up;
	unsigned int port, bps, cpu_hz;

	board_consoleinfo_get(&port, &bps, &cpu_hz);

	up = (smc_uart_t *)&CPMP->cp_dparam[mpc8xx_tty_proff[port]];
	tbdf = (cbd_t *)&CPMP->cp_dpmem[up->smc_tbase];

	/* Wait for last character to go.
	*/
	buf = (char *)tbdf->cbd_bufaddr;

	do {
	  mpc8xx_watchdog_kick();
	} 
	while (tbdf->cbd_sc & BD_SC_READY);

	*buf = c;
	tbdf->cbd_datlen = 1;
	tbdf->cbd_sc |= BD_SC_READY;
}

char
serial_getc()
{
	volatile cbd_t		*rbdf;
	volatile char		*buf;
	volatile smc_uart_t	*up;
	char			c;
	unsigned int port, bps, cpu_hz;

	board_consoleinfo_get(&port, &bps, &cpu_hz);

	up = (smc_uart_t *)&CPMP->cp_dparam[mpc8xx_tty_proff[port]];

	/* figure out which bd that is the first one with data */
	
	rbdf = (cbd_t *)&CPMP->cp_dpmem[up->smc_rbase];
	rbdf += RING_CNT;

//	printf("waiting for data, bd = 0x%08x ", (u32) rbdf);

	/* Wait for character to show up.
	*/
	buf = (char *)rbdf->cbd_bufaddr;
	do {
	  mpc8xx_watchdog_kick();
	} while (rbdf->cbd_sc & BD_SC_EMPTY);

	c = *buf;

//	printf("received 0x%02x\n", c);

	rbdf->cbd_sc |= BD_SC_EMPTY;
	RING_CNT = (RING_CNT + 1) & (RX_NO_BDS - 1);

	return(c);
}

int
serial_tstc()
{
	volatile cbd_t		*rbdf;
	volatile smc_uart_t	*up;
	unsigned int port, bps, cpu_hz;

	board_consoleinfo_get(&port, &bps, &cpu_hz);
	mpc8xx_watchdog_kick();

	up = (smc_uart_t *)&CPMP->cp_dparam[mpc8xx_tty_proff[port]];

	/* check if the current bd has data, then we're full! */

	rbdf = (cbd_t *)&CPMP->cp_dpmem[up->smc_rbase];
	rbdf += RING_CNT;

	return !(rbdf->cbd_sc & BD_SC_EMPTY);
}

void
serial_shutdown(void)
{
	volatile cbd_t		*tbdf;
	volatile smc_uart_t	*up;
	volatile smc_t		*sp;
	volatile cpm8xx_t	*cp;
	uint port;
	int k;

	board_consoleinfo_get(&port, &k, &k);

	cp = CPMP;
	sp = (smc_t*)&(cp->cp_smc[port]);
	up = (smc_uart_t *)&cp->cp_dparam[mpc8xx_tty_proff[port]];
	tbdf = (cbd_t *)&cp->cp_dpmem[up->smc_tbase];

	/* Wait for last character to go.
	*/

	do {
	  mpc8xx_watchdog_kick();
	} 
	while (tbdf->cbd_sc & BD_SC_READY);


	/* wait until all characters are sent through the fifo */
	/* the slowest speed we support is 9600                */

	for (k = 0; k < 2; k++)
	  cpu_udelay((1000000 / 9600) * 10);


	/* Send the CPM a stop transmit command.
	 */
	
	cp->cp_cpcr = mk_cr_cmd(mpc8xx_tty_cpmch[port], 
				CPM_CR_STOP_TX) | CPM_CR_FLG;
	while (cp->cp_cpcr & CPM_CR_FLG);


	/* Disable transmitter/receiver.
	*/
	sp->smc_smcmr &= ~(SMCMR_REN | SMCMR_TEN);


	/* do NOT reset the entire CPM here,
	 * that will modify the parameter ram.
	 */


	/* Restore Port B.
	*/

	if (port == 0) {
	      cp->cp_pbpar &= ~0x000000c0;      /* SMC1 */
	      cp->cp_pbdir |= 0x000000c0;
	      cp->cp_pbodr |= 0x000000c0;
	}
	else {
	      cp->cp_pbpar &= ~0x00000c00;      /* SMC2 */
	      cp->cp_pbdir |= 0x00000c00;
	      cp->cp_pbodr |= 0x00000c00;
	}
}






















