/* ------------------------------------------------------------------------- */
/* arch/ppc/mpc8xx/start.S                                                   */
/*                                                                           */
/* mpc8xx specific boot and lowlevel code                                    */
/*                                                                           */
/* Dan Malek <dmalek@jlc.net>                                                */
/* Marcus Sundberg <marcus@cendio.se>                                        */
/* Magnus Damm <damm@opensource.se>                                          */
/*                                                                           */
/* Bit definitions stolen from the Linux kernel.                             */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston,                     */
/* MA 02111-1307 USA                                                         */
/* ------------------------------------------------------------------------- */

#define __ASSEMBLY__
	
#include "../ppc_asm.tmpl"

#define _GLOBAL(n)\
	.globl n;\
n:
	
/* SPR definitions for 860 core */
#define SSR1	 27
#define ICR	148
#define DER	149
#define ICTRL	158
#define PVR	287
#define IC_CST	560
#define DC_CST	568
#define IMMR	638


/* cache bitfield definitions */
#define CST_UNLOCK_ALL		0x0a000000
#define CST_INVALIDATE_ALL	0x0c000000
#define CST_CACHE_ENABLE	0x02000000
#define CST_CACHE_DISABLE	0x04000000


/* SPR init values */
#define MSR_VALUE	0x00000000
#define ICTRL_VALUE	7

	.text
        .ascii   "MINIBOOT"
        .align 2

_GLOBAL(mpc8xx_image_start_now)
	/* Invalidate instruction cache */
  	lis	r2, CST_INVALIDATE_ALL@h 
	mtspr   IC_CST, r2

	/* Turn off instruction cache */
	lis	r2, CST_CACHE_DISABLE@h 
	mtspr   IC_CST, r2
	
	mtlr	r3              /* start address */

	li	r31, 0
	mr	r0, r31
	mr	r1, r31
	mr	r2, r31
	mr	r3, r31
	mr	r4, r31
	mr	r5, r31
	mr	r6, r31
	mr	r7, r31
	mr	r8, r31
	mr	r9, r31
	mr	r10, r31
	mr	r11, r31
	mr	r12, r31
	mr	r13, r31
	mr	r14, r31
	mr	r15, r31
	mr	r16, r31
	mr	r17, r31
	mr	r18, r31
	mr	r19, r31
	mr	r20, r31
	mr	r21, r31
	mr	r22, r31
	mr	r23, r31
	mr	r24, r31
	mr	r25, r31
	mr	r26, r31
	mr	r27, r31
	mr	r28, r31
	mr	r29, r31
	mr	r30, r31
	blr

_GLOBAL(init)
	
	/* Unlock and invalidate caches. */
  	lis	r3, CST_UNLOCK_ALL@h 
	mtspr   IC_CST, r3
	mtspr   DC_CST, r3

  	lis	r3, CST_INVALIDATE_ALL@h 
	mtspr   IC_CST, r3
	mtspr   DC_CST, r3

	/* Enable instruction cache. */
	lis	r3, CST_CACHE_ENABLE@h 
	mtspr   IC_CST, r3
	
	/* Disable data cache. */
	lis	r3, CST_CACHE_DISABLE@h 
	mtspr   DC_CST, r3

	/* Invalidate all TLB's */
	tlbia
	isync
	
	/* Initialize some sprs that are hard to access from C */
	li	r3,ICTRL_VALUE 
	mtspr	ICTRL,r3

	/* set ICR and DER to 0 */
	
	li	r3,0             
	mtspr	ICR,r3
	mtspr	DER,r3

	/* Jump to C-code */
	b	main	

	. = 0x100

_GLOBAL(start)
_GLOBAL(mpc8xx_start)
#ifdef BOARD_IS_ODEN

        /* to support old bootloader software, we need to have */
        /* a space between 0x110 and 0x1000. */
        
        b       foo

        . = 0x110
        
        .fill 0xef0, 1, 0xff
        
        . = 0x1000
foo:    
#endif  
       
	/* First of all - set MSR to a defined value. */
	lis	r3,MSR_VALUE@h 
	ori	r3,r3,MSR_VALUE@l
	mtmsr	r3
	mtspr	SSR1,r3 

	mfspr	r1, IMMR		/* get IMMR register       */ 
	andis.	r1, r1, 0xffff0000@h    /* we only want the hi16   */
					/* setup stack pointer to  */  
	ori	r1, r1, 0x2ff0          /* top of dual port memory */
	b	init

_GLOBAL(mpc8xx_icache_invalidate)
  	lis	r3, CST_INVALIDATE_ALL@h 
	mtspr   IC_CST, r3
	isync
	sync
	blr
	


