/* ------------------------------------------------------------------------- */
/* board/corelatus_gth/board.c                                               */
/*                                                                           */
/* Board specific code for GTH. Based on FADS                                */
/*                                                                           */
/* Based on "fadsrom.tgz", written by Dan Malek.                             */
/* Modified by Magnus Damm <damm@opensource.se>                              */
/* Adapted by Thomas Lange <thomas@corelatus.se>                             */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston,                     */
/* MA 02111-1307 USA                                                         */
/* ------------------------------------------------------------------------- */

#include "types.h"
#include "board.h"
#include "cpu.h"
#include "console.h"

#include "8xx_immap.h"

#define CONSOLE_PORT 0                   /* use SMC1 */
//#define CONSOLE_PORT 1                   /* use SMC2 */

void board_flash_program_init(void);
void board_flash_setup(unsigned int bcsr2);

/* ------------------------------------------------------------------------- */
/* board_init()                                                              */
/* this code is called early, before console is present.                     */

void board_init(char **str)
{

 /* kick the dog! */

  mpc8xx_watchdog_kick();


  /* Init PLL */
  
  mpc8xx_set_pll(CPU_XFACT);


  *str = ", GTH Board";


  /* copy the flash burning code to dpram */

  board_flash_program_init();

  /* kick again */

  mpc8xx_watchdog_kick();


#if 0
  /* figure out the size of the flash SIMM */

  {
    volatile immap_t *immap = mpc8xx_get_imap();
    u32 tmp_or0;
    volatile u32 *bcsr;

    /* set CS0 to 8MByte */

    tmp_or0 = immap->im_memctl.memc_or0;

    immap->im_memctl.memc_br0 |= 1;
    immap->im_memctl.memc_or0 = immap->im_memctl.memc_or0 | 0xff800000;
    immap->im_memctl.memc_br0 |= 1;


    mpc8xx_watchdog_kick();

    bcsr = (u32 *) 0x800000;

    board_flash_setup(bcsr[2]);

    /* enable the console */

#if CONSOLE_PORT == 0
    bcsr[1] &= ~0x01000000; /* BCSR1_RS232EN_1 */
#else
    bcsr[1] &= ~0x00040000; /* BCSR1_RS232EN_2 */
#endif

    /* wait a while for some external wierd hardware */

    cpu_udelay(1000);

    /* remove CS1 from space and restore CS0 */

    immap->im_memctl.memc_br1 = 0;
    
    immap->im_memctl.memc_br0 |= 1;
    immap->im_memctl.memc_or0 = tmp_or0;
    immap->im_memctl.memc_br0 |= 1;

  }
#endif
}

void board_cleanup(void)
{
  /* kick the dog! */

  mpc8xx_watchdog_kick();
#if 0
  {
    volatile immap_t *immap = mpc8xx_get_imap();
    u32 tmp_or0;
    volatile u32 *bcsr;

    /* set CS0 to 8MByte */

    tmp_or0 = immap->im_memctl.memc_or0;

    immap->im_memctl.memc_br0 |= 1;
    immap->im_memctl.memc_or0 = immap->im_memctl.memc_or0 | 0xff800000;
    immap->im_memctl.memc_br0 |= 1;

    /* map up BCSR (CS1) at 8MByte */

    immap->im_memctl.memc_or1 = 0xffff8110;
    immap->im_memctl.memc_br1 = 0x00800001;

    bcsr = (u32 *) 0x800000;

    /* turn off console */

#if CONSOLE_PORT == 0
    bcsr[1] |= 0x01000000; /* BCSR1_RS232EN_1 */
#else
    bcsr[1] |= 0x00040000; /* BCSR1_RS232EN_2 */
#endif

    /* remove CS1 from space and restore CS0 */

    immap->im_memctl.memc_br1 = 0;
    
    immap->im_memctl.memc_br0 |= 1;
    immap->im_memctl.memc_or0 = tmp_or0;
    immap->im_memctl.memc_br0 |= 1;

  }
#endif
}

/* ------------------------------------------------------------------------- */
/* serial port                                                               */

void board_consoleinfo_get(unsigned int *port, 
				   unsigned int *bps,
				   unsigned int *cpu_hz)
{
  *port = CONSOLE_PORT;
  *bps = 9600;
  *cpu_hz = CPU_HZ;
}

/* ------------------------------------------------------------------------- */
