/* ------------------------------------------------------------------------- */
/* lib/flash.c                                                               */
/*                                                                           */
/* generic flash code for Miniboot.                                          */
/*                                                                           */
/* Copyright (C) 2000, 2001, 2002 Magnus Damm <damm@opensource.se>           */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston,                     */
/* MA 02111-1307 USA                                                         */
/* ------------------------------------------------------------------------- */

#include "cpu.h"
#include "console.h"
#include "board.h"
#include "flash.h"
#include "string.h"

int flash_verbose(int ret)
{
  if (ret == FLASH_OK) 
    return 0;
  else
    printf("Flash error: ");

  switch(ret) {
    
  case FLASH_BAD_ADDRESS:
    printf("Address error.\n");
    break;
    
  case FLASH_BAD_DEVICE_ID:
    printf("Bad device id.\n");
    break;
    
  case FLASH_BAD_MANUFACTURER_ID:
    printf("Bad manufacturer id.\n");
    break;

  case FLASH_ERASE_ERROR:
    printf("Erase error.\n");
    break;

  case FLASH_PROGRAM_ERROR:
    printf("Program error.\n");
    break;

  case FLASH_SECTOR_PROTECTED_ERROR:
    printf("Sector protected error.\n");
    break;
    
  case FLASH_ERROR:
  default:
    printf("unknown error.\n");
  }

  return -1;
}

int flash_erase_sector(unsigned int sector)
{
  unsigned int s = 1;

#ifndef FLASH_SLOW_ERASE

  unsigned int k, n;
  unsigned int *base;

  board_flash_getsectorinfo(sector, (unsigned char **)&base, &n);

  /* look through the sector to see if we need to erase it */
  /* this might make erase of large flashes much faster.. */

  if (!(((unsigned int)base & 0x03) || (n & 0x03))) {
    s = k = 0;
    n >>= 2;

    while ((s == 0) && (k < n)) {
      if ((k & 0xffff) == 0) {
	cpu_watchdog_kick();
      }    

      s = (base[k] != 0xffffffff);
      k++;
    }
  }

#endif

  cpu_watchdog_kick();

  if (s) {
	  s = board_flash_erase_sector(sector);
  }
  else {
	  s = FLASH_OK;
  }

  return s;
}


