#!/bin/sh
#
# convert a binary file into a "flat binary file" 
# used by ESTs visionPROBE
#
# written by Magnus Damm
#

function echo_hex_8 () {
    o3=`echo $(( $1 / 64 ))`
    o2=`echo $(( ($1 - ($o3 * 64)) / 8 ))`
    o1=`echo $(( ($1 - ($o3 * 64)) - ($o2 * 8) ))`
#    echo "$1 -  $o3 $o2 $o1"
    
    echo -ne "\\$o3$o2$o1";
}


function echo_hex_32 () {

    val1=$(( $1 / 0x1000000 ))
    val2=$(( ($1 - ($val1 * 0x1000000)) / 0x10000 ))
    val3=$(( (($1 - ($val1 * 0x1000000)) - ($val2 * 0x10000)) / 0x100 ))
    val4=$(( ((($1 - ($val1 * 0x1000000)) - ($val2 * 0x10000)) \
		   - ($val3 * 0x100)) ))

#    echo "$1: $val1 $val2 $val3 $val4"

    echo_hex_8 $val1
    echo_hex_8 $val2
    echo_hex_8 $val3
    echo_hex_8 $val4;
}


if [ $# -lt 3 ]; then
    echo "Usage: make_flatbin.sh sourcefile destinationfile startaddress"
    exit 1
fi

src=$1
dst=$2
startaddr=$3

wcout=`wc -c $src`
srcsize=`echo $wcout | cut -d " " -f 1`

# the EST software (runs on windows.. no wonder it sucks)
#
# will program wierd data in the last bytes of your flash
# if ((size_of_file & 0x03) != 0)
#
# so, add bytes to the file if needed.

needs_add=`echo $(( $srcsize - (($srcsize / 4) * 4) ))`

endaddr=`echo $(( $startaddr + $srcsize + 1 + $needs_add))`

printf "0x%08x -> 0x%08x (0x%08x bytes)\n" $startaddr $endaddr $srcsize

echo -n "ESTFBINR" > $dst
echo_hex_32 $startaddr >> $dst
echo_hex_32 $endaddr >> $dst
echo_hex_32 0 >> $dst
echo_hex_32 0 >> $dst
echo_hex_32 0 >> $dst
echo_hex_32 0 >> $dst

cat $src >> $dst

if [ $needs_add -eq 0 ]; then
    exit 0;
fi

echo "Adding $needs_add bytes."
    
n=0
while [ $n -lt $needs_add ];
do
    echo_hex_8 0 >> $dst
    n=$(( $n + 1 ))
done










